-----------------------------------------
-- Database of all minipets and mounts --
-----------------------------------------

--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local MAJOR_VERSION = "PetAndMountDatabase-1.0";
local MINOR_VERSION = 20402;

if not AceLibrary then
   error( MAJOR_VERSION .. " requires AceLibrary" );
end
if not AceLibrary:IsNewVersion( MAJOR_VERSION, MINOR_VERSION ) then
   return;
end

local pt3 = LibStub( "LibPeriodicTable-3.1", true );

local getItemID;

function getItemID( item )
    -- accepts either an item string ie "item:12345:0:0:0:2342:123324:12:1", hyperlink, or an itemid.
    -- returns a number'ified itemid.
    return tonumber( item ) or tonumber( item:match( "item:(%d+)") ) or ( -1 * ( ( item:match( "enchant:(%d+)" ) or item:match( "spell:(%d+)" ) ) or 0 ) );
end

local PetAndMountDatabase =
{
    Version = MINOR_VERSION, -- Matches the WoW version number this is accurate for
    Collections = {
        MiniPets = {
            Normal = {
                [19450] = true, -- A Jubling's Tiny Home
                [11023] = true, -- Ancona Chicken
                [34535] = true, -- Azure Whelpling
                [32588] = true, -- Banana Charm
                [10360] = true, -- Black Kingsnake
                [29958] = true, -- Blue Dragonhawk Hatchling
                [29901] = true, -- Blue Moth Egg
                [20371] = true, -- Blue Murloc Egg
                [29364] = true, -- Brown Rabbit Crate
                [10361] = true, -- Brown Snake
                [29960] = true, -- Captured Firefly
                [23083] = true, -- Captured Flame
                [8491]  = true, -- Cat Carrier (Black Tabby)
                [8485]  = true, -- Cat Carrier (Bombay)
                [8486]  = true, -- Cat Carrier (Cornish Rex)
                [8487]  = true, -- Cat Carrier (Orange Tabby)
                [8490]  = true, -- Cat Carrier (Siamese)
                [8488]  = true, -- Cat Carrier (Silver Tabby)
                [8489]  = true, -- Cat Carrier (White Kitten)
                [11110] = true, -- Chicken Egg
                [35350] = true, -- Chuck's Bucket
                [34425] = true, -- Clockwork Rocket Bot
                [10393] = true, -- Cockroach
                [10392] = true, -- Crimson Snake
                [10822] = true, -- Dark Whelpling
                [34493] = true, -- Dragon Kite
                [13584] = true, -- Diablo Stone
                [20769] = true, -- Disgusting Oozeling
                [32616] = true, -- Egbert's Egg
                [32622] = true, -- Elekk Training Collar
                [29953] = true, -- Golden Dragonhawk Hatchling
                [8500]  = true, -- Great Horned Owl
                [8501]  = true, -- Hawk Owl
                [23713] = true, -- Hippogryph Hatchling
                [15996] = true, -- Lifelike Mechanical Toad
                [11826] = true, -- Lil' Smoky
                [30360] = true, -- Lurky's Egg
                [27445] = true, -- Magical Crawdad Box
                [29363] = true, -- Mana Wyrmling
                [10398] = true, -- Mechanical Chicken
                [4401]  = true, -- Mechanical Squirrel Box
                [31760] = true, -- Miniwing
                [33993] = true, -- Mojo
                [33818] = true, -- Muckbreath's Bucket
                [25535] = true, -- Netherwhelp's Collar
                [13583] = true, -- Panda Collar
                [8496]  = true, -- Parrot Cage (Cockatiel)
                [8492]  = true, -- Parrot Cage (Green Wing Macaw)
                [8494]  = true, -- Parrot Cage (Hyacinth Macaw)
                [8495]  = true, -- Parrot Cage (Senegal)
                [11825] = true, -- Pet Bombling
                [35504] = true, -- Phoenix Hatchling
                [23007] = true, -- Piglet's Collar
                [22114] = true, -- Pink Murloc Egg
                [10394] = true, -- Prairie Dog Whistle
                [8497]  = true, -- Rabbit Crate (Snowshoe)
                [23015] = true, -- Rat Cage
                [29956] = true, -- Red Dragonhawk Hatchling
                [29902] = true, -- Red Moth Egg
                [34492] = true, -- Rocket Chicken
                [34955] = true, -- Scorched Stone
                [29957] = true, -- Silver Dragonhawk Hatchling
                [33154] = true, -- Sinister Squashling
                [32617] = true, -- Sleepy Willy
                [12529] = true, -- Smolderweb Carrier
                [35349] = true, -- Snarly's Bucket
                [11474] = true, -- Sprite Darter Egg
                [8499]  = true, -- Tiny Crimson Whelpling
                [8498]  = true, -- Tiny Emerald Whelpling
                [34478] = true, -- Tiny Spore Bat
                [33816] = true, -- Toothy's Bucket
                [21277] = true, -- Tranquil Mechanical Yeti
                [11026] = true, -- Tree Frog Box
                [22235] = true, -- Truesilver Shafted Arrow
                [23002] = true, -- Turtle Box
                [29904] = true, -- White Moth Egg
                [12264] = true, -- Worg Carrier
                [32233] = true, -- Wolpertinger's Tankard
                [29903] = true, -- Yellow Moth Egg
                [13582] = true, -- Zergling Leash
                }, -- Normal
            
            Holiday = { -- Needs a snowball
                [21301] = true, -- Green Helper Box
                [21308] = true, -- Jingling Bell
                [21305] = true, -- Red Helper Box
                [21309] = true, -- Snowman Kit
                }, -- Holiday

            Equipment = { -- Needs to be equipped
                [12185] = true,  -- Bloodsail Admiral's Hat
                }, -- Equipment
    
            Children = { -- Children's week
                [31880] = true, -- Blood Elf Orphan Whistle
                [31881] = true, -- Draenei Orphan Whistle
                [18598] = true, -- Human Orphan Whistle
                [18597] = true, -- Orcish Orphan Whistle
                }, -- Children
                
            Quest = { -- Used in a quest
                [34253] = true, -- Sizzling Embers
                }, -- Quest
        }, -- MiniPets
        
        Mounts = {
            Ground = {
                Slow = { -- +0% speed
                    [33189] = 0, -- Rickety Magic Broom
                    [23720] = 0, -- Riding Turtle
                    }, -- Slow
                
                Fast = { -- +60% speed
                    [29221] = 60, -- Black Hawkstrider
                    [29220] = 60, -- Blue Hawkstrider
                    [8595]  = 60, -- Blue Mechanostrider
                    [13332] = 60, -- Blue Skeletal Horse
                    [2411]  = 60, -- Black Stallion Bridle
                    [33976] = 60, -- Brewfest Ram
                    [5656]  = 60, -- Brown Horse Bridle
                    [13333] = 60, -- Brown Skeletal Horse
                    [28481] = 60, -- Brown Elekk
                    [15290] = 60, -- Brown Kodo
                    [5872]  = 60, -- Brown Ram
                    [5655]  = 60, -- Chestnut Mare Bridle
                    [29744] = 60, -- Gray Elekk
                    [15277] = 60, -- Gray Kodo
                    [5864]  = 60, -- Gray Ram
                    [3321]  = 60, -- Green Mechanostrider
                    [5668]  = 60, -- Horn of the Brown Wolf
                    [5665]  = 60, -- Horn of the Dire Wolf
                    [1132]  = 60, -- Horn of the Timber Wolf
                    [33183] = 60, -- Magic Broom
                    [2414]  = 60, -- Pinto Bridle
                    [29743] = 60, -- Purple Elekk
                    [29222] = 60, -- Purple Hawkstrider
                    [28927] = 60, -- Red Hawkstrider
                    [8563]  = 60, -- Red Mechanostrider
                    [13331] = 60, -- Red Skeletal Horse
                    [33224] = 60, -- Reins of the Spectral Tiger
                    [8632]  = 60, -- Reins of the Spotted Frostsaber
                    [8631]  = 60, -- Reins of the Striped Frostsaber
                    [8628]  = 60, -- Reins of the Spotted Nightsaber
                    [8629]  = 60, -- Reins of the Striped Nightsaber
                    [13322] = 60, -- Unpainted Mechanostrider
                    [8588]  = 60, -- Whistle of the Emerald Raptor
                    [8591]  = 60, -- Whistle of the Turquoise Raptor
                    [8592]  = 60, -- Whistle of the Violet Raptor
                    [5873]  = 60, -- White Ram
                    }, -- Fast
                
                VeryFast = { -- +100% speed
                    [33809] = 100, -- Amani War Bear
                    [18243] = 100, -- Black Battlestrider
                    [29256] = 100, -- Black Battlestrider
                    [21176] = 100, -- Black Qiraji Resonating Crystal
                    [13328] = 100, -- Black Ram
                    [18247] = 100, -- Black War Kodo
                    [29466] = 100, -- Black War Kodo
                    [18244] = 100, -- Black War Ram
                    [29467] = 100, -- Black War Ram
                    [18241] = 100, -- Black War Steed Bridle
                    [29468] = 100, -- Black War Steed Bridle
                    [13335] = 100, -- Deathcharger's Reins
                    [30480] = 100, -- Fiery Warhorse's Reins
                    [13329] = 100, -- Frost Ram
                    [29745] = 100, -- Great Blue Elekk
                    [18794] = 100, -- Great Brown Kodo
                    [18795] = 100, -- Great Gray Kodo
                    [29746] = 100, -- Great Green Elekk
                    [29747] = 100, -- Great Purple Elekk
                    [18793] = 100, -- Great White Kodo
                    [15292] = 100, -- Green Kodo
                    [13334] = 100, -- Green Skeletal Warhorse
                    [12351] = 100, -- Horn of the Arctic Wolf
                    [18245] = 100, -- Horn of the Black War Wolf
                    [29469] = 100, -- Horn of the Black War Wolf
                    [19029] = 100, -- Horn of the Frostwolf Howler
                    [12330] = 100, -- Horn of the Red Wolf
                    [18796] = 100, -- Horn of the Swift Brown Wolf
                    [18798] = 100, -- Horn of the Swift Gray Wolf
                    [18797] = 100, -- Horn of the Swift Timber Wolf
                    [13327] = 100, -- Icy Blue Mechanostrider Mod A
                    [12354] = 100, -- Palomino Bridle
                    [18791] = 100, -- Purple Skeletal Warhorse
                    [29470] = 100, -- Red Skeletal Warhorse
                    [35906] = 100, -- Reins of the Black War Elekk
                    [18242] = 100, -- Reins of the Black War Tiger
                    [29471] = 100, -- Reins of the Black War Tiger
                    [31829] = 100, -- Reins of the Cobalt Riding Talbuk
                    [31830] = 100, -- Reins of the Cobalt Riding Talbuk
                    [29102] = 100, -- Reins of the Cobalt War Talbuk
                    [29227] = 100, -- Reins of the Cobalt War Talbuk
                    [28915] = 100, -- Reins of the Dark Riding Talbuk
                    [29228] = 100, -- Reins of the Dark War Talbuk
                    [12302] = 100, -- Reins of the Frostsaber
                    [12303] = 100, -- Reins of the Nightsaber
                    [32768] = 100, -- Reins of the Raven Lord
                    [31831] = 100, -- Reins of the Silver Riding Talbuk
                    [31832] = 100, -- Reins of the Silver Riding Talbuk
                    [29104] = 100, -- Reins of the Silver War Talbuk
                    [29229] = 100, -- Reins of the Silver War Talbuk
                    [18766] = 100, -- Reins of the Swift Frostsaber
                    [18767] = 100, -- Reins of the Swift Mistsaber
                    [33225] = 100, -- Reins of the Swift Spectral Tiger
                    [18902] = 100, -- Reins of the Swift Stormsaber
                    [31833] = 100, -- Reins of the Tan Riding Talbuk
                    [31834] = 100, -- Reins of the Tan Riding Talbuk
                    [29105] = 100, -- Reins of the Tan War Talbuk
                    [29230] = 100, -- Reins of the Tan War Talbuk
                    [13086] = 100, -- Reins of the Winterspring Frostsaber
                    [31835] = 100, -- Reins of the White Riding Talbuk
                    [31836] = 100, -- Reins of the White Riding Talbuk
                    [29103] = 100, -- Reins of the White War Talbuk
                    [29231] = 100, -- Reins of the White War Talbuk
                    [19030] = 100, -- Stormpike Battle Charger
                    [18788] = 100, -- Swift Blue Raptor
                    [33977] = 100, -- Swift Brewfest Ram
                    [18786] = 100, -- Swift Brown Ram
                    [18777] = 100, -- Swift Brown Steed
                    [18787] = 100, -- Swift Gray Ram
                    [29223] = 100, -- Swift Green Hawkstrider
                    [18772] = 100, -- Swift Green Mechanostrider
                    [33184] = 100, -- Swift Magic Broom
                    [18789] = 100, -- Swift Olive Raptor
                    [18790] = 100, -- Swift Orange Raptor
                    [18776] = 100, -- Swift Palomino
                    [28936] = 100, -- Swift Pink Hawkstrider
                    [29224] = 100, -- Swift Purple Hawkstrider
                    [19872] = 100, -- Swift Razzashi Raptor
                    [34129] = 100, -- Swift Warstrider
                    [35513] = 100, -- Swift White Hawkstrider
                    [18773] = 100, -- Swift White Mechanostrider
                    [18785] = 100, -- Swift White Ram
                    [18778] = 100, -- Swift White Steed
                    [18774] = 100, -- Swift Yellow Mechanostrider
                    [19902] = 100, -- Swift Zulian Tiger
                    [15293] = 100, -- Teal Kodo
                    [18246] = 100, -- Whistle of the Black War Raptor
                    [29472] = 100, -- Whistle of the Black War Raptor
                    [13317] = 100, -- Whistle of the Ivory Raptor
                    [8586]  = 100, -- Whistle of the Mottled Red Raptor
                    [13326] = 100, -- White Mechanostrider Mod A
                    [12353] = 100, -- White Stallion Bridle
                    }, -- VeryFast
                }, -- Ground
            Flying = {
                Fast = { -- +60% speed
                    [25475] = 60, -- Blue Windrider
                    [25471] = 60, -- Ebon Gryphon
                    [33176] = 60, -- Flying Broom
                    [34060] = 60, -- Flying Machine Control
                    [25470] = 60, -- Golden Gryphon
                    [25476] = 60, -- Green Windrider
                    [25472] = 60, -- Snowy Gryphon
                    [25474] = 60, -- Tawny Windrider
                    [35225] = 60, -- X-51 Nether-Rocket
                    }, -- Fast
                
                VeryFast = { -- +280% speed
                    [32319] = 280, -- Blue Riding Nether Ray
                    [33999] = 280, -- Cenarion War Hippogryph
                    [32314] = 280, -- Green Riding Nether Ray
                    [32316] = 280, -- Purple Riding Nether Ray
                    [32317] = 280, -- Red Riding Nether Ray
                    [32858] = 280, -- Reins of the Azure Netherwing Drake
                    [32859] = 280, -- Reins of the Cobalt Netherwing Drake
                    [32857] = 280, -- Reins of the Onyx Netherwing Drake
                    [32860] = 280, -- Reins of the Purple Netherwing Drake
                    [32861] = 280, -- Reins of the Veridian Netherwing Drake
                    [32862] = 280, -- Reins of the Violet Netherwing Drake
                    [32318] = 280, -- Silver Riding Nether Ray
                    [25473] = 280, -- Swift Blue Gryphon
                    [33182] = 280, -- Swift Flying Broom
                    [25528] = 280, -- Swift Green Gryphon
                    [25531] = 280, -- Swift Green Windrider
                    [25529] = 280, -- Swift Purple Gryphon
                    [25533] = 280, -- Swift Purple Windrider
                    [25527] = 280, -- Swift Red Gryphon
                    [25477] = 280, -- Swift Red Windrider
                    [25532] = 280, -- Swift Yellow Windrider
                    [34061] = 280, -- Turbo-Charged Flying Machine Control
                    [35226] = 280, -- X-51 Nether-Rocket XTREME
                    }, -- VeryFast
                
                ExtremelyFast = { -- +310% speed
                    [32458] = 310, -- Ashes of Al'ar
                    [34092] = 310, -- Merciless Nether Drake
                    [30609] = 310, -- Swift Nether Drake
                    }, -- ExtremelyFast
            }, -- Flying
        }, -- Mounts
    }, -- Collections
};

-- Build PT3 like set names
PetAndMountDatabase.Sets = {};
for setName, set in pairs( PetAndMountDatabase.Collections ) do
    for subsetName, subset in pairs( set ) do
        for additionsetName, additionalset in pairs( subset ) do
            if( type( additionalset ) == "table" ) then
                PetAndMountDatabase.Sets[setName .. "." .. subsetName .. "." .. additionsetName] = additionalset;
            else
                PetAndMountDatabase.Sets[setName .. "." .. subsetName] = subset;
                break;
            end --if
        end --for
    end --for
end --for

if( pt3 ) then
    -- Check to see if we want to use pt for some sets
    function checkSet( set, pt3Set, setName )
        local Complete = true;
        local returnSet = set;
        
        for itemId, _ in pairs( set ) do
            if( not pt3:ItemInSet( itemId, pt3Set ) ) then
                Complete = false;
            end --if
        end --for
        if( Complete ) then
            -- the pt3 set is at least as complete as ours
            PetAndMountDatabase.Sets[setName] = pt3Set;
            returnSet = pt3:GetSetTable( pt3Set );
        end --if
        return( returnSet );
    end --checkSet( set, pt3Set )

    PetAndMountDatabase.Collections.MiniPets.Normal = checkSet( PetAndMountDatabase.Collections.MiniPets.Normal, "Misc.Minipet.Normal", "MiniPets.Normal" );
    PetAndMountDatabase.Collections.MiniPets.Holiday = checkSet( PetAndMountDatabase.Collections.MiniPets.Holiday, "Misc.Minipet.Snowball", "MiniPets.Holiday" );
end --if

function PetAndMountDatabase:ItemInSet( item, set )
    if( item ~= nil ) then
        assert( type( item ) == "number" or type( item ) == "string", "Invalid arg1: item must be a number or item link" );
        assert( type( set ) == "string", "Invalid arg2: set must be a string" );
        -- Type the passed item out to an itemid.
        item = getItemID( item );
        assert( item ~= 0, "Invalid arg1: invalid item." );
        
        local setUse = self.Sets[set];

        assert( setUse, "Invalid arg2: invalid set." );
    
        if( type( setUse == "table" ) ) then
            return setUse[item];
        elseif( pt3 ) then
            return pt3:ItemInSet( item, setUse );
        end --if
    end
    return false;
end --PetAndMountDatabase:ItemInSet( item, set )

local function activate( self, oldLib, oldDeactivate )
    if( oldLib ) then -- if upgrading
        self.Collections = oldLib.Collections;
        self.Version = oldLib.Version;
        self.Sets = oldLib.Sets;
    end --if
    if oldDeactivate then -- clean up the old library
        oldDeactivate( oldLib );
    end --if
end --activate( self, oldLib, oldDeactivate )

AceLibrary:Register( PetAndMountDatabase, MAJOR_VERSION, MINOR_VERSION, activate, nil, nil );